      program pingpong
      implicit none
c FORTRAN MPI-INCLUDE-file
      include 'mpif.h'
c-------------------------------------------------------------------+
C--      pingpong test to evaluate communication 
C--      between two processors
c--      If run on nproc processors, pingpong evaluates
c--      the communication between processor 0 and processor ip,
c--      ip = 1,...,nproc-1
c-------------------------------------------------------------------+
      integer status(MPI_STATUS_SIZE)
      integer maxlen, maxproc 
      parameter ( maxlen = 2000000 , maxproc = 128 )
      integer myid, numt, i, ierr, il, len, irep, lrep
      integer nproc, iw, ip
      integer ncom, rtag, stag, pnamelen
      double precision vec(maxlen), snd_vec(maxlen), rcv_vec(maxlen)
      double precision lat(maxproc), speedmax(maxproc)
      double precision t0, t1, tlat, speedg, tmin, tg, dt, speed 
      character*32 pname
      real timer
c---------------------------------------------------------- 
c        start MPI
c----------------------------------------------------------
      call MPI_INIT( ierr )
      call MPI_COMM_SIZE( MPI_COMM_WORLD, nproc, ierr )
      call MPI_COMM_RANK( MPI_COMM_WORLD, myid, ierr )
      if (nproc.gt.maxproc) then
         if (myid.eq.0) then
            write(6,*)  'nproc = ',nproc,' too large'
            write(6,*)  'maximal nproc is ',maxproc
         end if
         stop
      end if
c----------------------------------------------------------
c        if first process, set environment
c----------------------------------------------------------
C      if (myid.eq.0) then
c-------------------------------------------------------------------+
c             Input 
c-------------------------------------------------------------------+
         iw =20 
! read(5,*) iw     number of different data sizes
         tlat = 0.0000005 
! read(5,*) tlat   expected latency per processo
         speedg = 1.d+9 
! read(5,*) speedg expected bandwidth between two processors
         tmin = 0.1
! read(5,*) tmin   expected time for measuring one data size

      if (myid.eq.0) then
         write(6,1001)nproc

 1001    format(/
     :'#    pingpong commumication test'/
     :'#    on ',i3,' processors')
         write(6,999)
 999     format(//
     :'#',2x,'    size    reps        ttot     latency     speed',/
     :'#',2x,' [Bytes]              [secs]      [secs]   [MB/secs]'/)
      end if

c-------------------------------------------------------------------+
c         communicate parameters 
c-------------------------------------------------------------------+
c      call MPI_BCAST(iw,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
c      call MPI_BCAST(tlat,1,MPI_DOUBLE_PRECISION,0,
c     :               MPI_COMM_WORLD,ierr)
c      call MPI_BCAST(speedg,1,MPI_DOUBLE_PRECISION,0,
c     :               MPI_COMM_WORLD,ierr)
c      call MPI_BCAST(tmin,1,MPI_DOUBLE_PRECISION,0,
c     :               MPI_COMM_WORLD,ierr)
      do ip = 1 , nproc - 1
         speedmax(ip) = 0.
         do 9 il = 0 , iw
            len = 2**il
            if (len.gt.maxlen) go to 9
            tg = 2*tlat + 16. * len / speedg
            lrep = 1 + tmin / tg
            do i = 1 , len
               snd_vec(i) = real(i)
               rcv_vec(i) = 0
            end do
c----------------------------------------------------------
c          allover synchronization 
c----------------------------------------------------------
            call MPI_BARRIER(MPI_COMM_WORLD,ierr) 
            t0 = MPI_WTIME() 
c----------------------------------------------------------
c          communicate data
c----------------------------------------------------------
            do irep = 1 , lrep
               stag = 1000 + 2*irep
               rtag = 1001 + 2*irep
               if (myid.eq.0) then
                  call MPI_SEND(snd_vec,len,MPI_DOUBLE_PRECISION,ip,
     :                          stag,MPI_COMM_WORLD,ierr) 
                  call MPI_RECV(rcv_vec,len,MPI_DOUBLE_PRECISION,ip,
     :                          rtag,MPI_COMM_WORLD,status,ierr) 
               else if(myid.eq.ip) then
                  call MPI_RECV(vec,len,MPI_DOUBLE_PRECISION,0,
     :                          stag,MPI_COMM_WORLD,status,ierr)
                  call MPI_SEND(vec,len,MPI_DOUBLE_PRECISION,0,
     :                          rtag,MPI_COMM_WORLD,ierr) 
               endif
            end do
c----------------------------------------------------------
            dt = MPI_WTIME() - t0

c----------------------------------------------------------
c         Test for correct communication 
c----------------------------------------------------------
            if (myid.eq.0) then
               do i = 1 , len
                  if (rcv_vec(i).ne.snd_vec(i)) then
                  write(6,*)
     :           'communication error ',i,rcv_vec(i),snd_vec(i)
                  go to 24
                  endif
               end do            
 24            continue
            endif
c----------------------------------------------------------
c         Output of communication speed 
c----------------------------------------------------------
            if (myid.eq.0) then
               if (dt.eq.0.) then
                  speed = 0.
               else
                  speed = 16.* len *lrep/ dt *1.e-6
               end if
               if (il.eq.0) lat(ip) = dt/real(2*lrep)
               if (speed.gt.speedmax(ip)) speedmax(ip) = speed 
               write(6,998)8*len,lrep,dt,dt/real(2*lrep),speed
 998           format(3x,2i8,2f12.7,f10.3)
            endif
 9       continue
      end do
      call MPI_BARRIER(MPI_COMM_WORLD,ierr) 
      call MPI_GET_PROCESSOR_NAME( pname, pnamelen, ierr )
      call MPI_BARRIER(MPI_COMM_WORLD,ierr)             
      if (myid.eq.0) then
         write(6,1002)nproc
 1002    format(//'#  pingpong summary'
     :           /'#  number of procs       : ',i3,//
     :           /'#  communication between proc 0 and proc ip:'/
     :           /'#  ip       latency [s]       bandwidth [GB/s]')
         do ip = 1 , nproc-1 
            write(6,1003) ip, lat(ip), speedmax(ip)/1000.
 1003       format('#  ',i3,8x,e9.2,8x,f8.2)
         end do
      end if

      call MPI_FINALIZE (ierr)

      stop
      end






















