      program demo_PDGESV
!     ==================
!  +-------------------------------------------------------------------+
!  |  Demo program for using the ScaLAPACK routine PDGESV              |
!  |  solving a system of linear equation with one right hand sides.   |
!  |  The following parameters are used:                               |
!  |  m_A = n, n_A = n, mb_A = nb, nb_A = nb, i_A = 1, j_A = 1         |
!  |  m_B = n, n_B = 1, mb_B = nb, nb_B = 1,  i_B = 1, j_B = 1         |
!  |  csrc_A = 0, rsrc_A= 0,  csrc_B = 0, rsrc_B= 0                    |
!  |  The matrices A and B are chosen as                               |
!  |  A(i,j) = 1./(mati + matj*abs(i-j), B(i,1) = veci*i + vecj        |
!  +-------------------------------------------------------------------+
!
      implicit none 

!  parameter constants
      integer           max_al, max_bl, max_vl
      parameter        (max_al=10000, max_bl=10000,  max_vl=10000)
!  variables for definition of matrix A and matrix X
      real*8 mati,matj,bi,bj
      common/matdat/mati,matj,bi,bj
!  local variables
      real*8    al(max_al), al_s(max_al), bl(max_bl), rl(max_bl),
     :          matval, bval, err
      integer   ctxt, ctxt_sys, ctxt_all,
     :          nproc, nprow, npcol, myid, myrow, mycol,
     :          m_al, n_al, m_bl, n_bl, m_cl, 
     :          llda, lldb, lldc, desc_A(9), desc_B(9), desc_R(9),
     :          n, nb, ncheck, 
     :          ipr, ipc, il, jl, i, j, info, vl(max_vl),  
     :          NUMROC, INDXL2G, INDXG2L, INDXG2P
!  calling BLACS_PINFO(myid,nproc) initializes MPI and returns size and rank
      call BLACS_PINFO(myid,nproc) 
      call BLACS_GET( 0, 0, ctxt_sys )
      ctxt_all = ctxt_sys
      call BLACS_GRIDINIT( ctxt_all, 'C', nproc, 1)
! input parameters
! Parameters for the distributed global matrices
      n = 9; nb = 2
! Parameters for processor grid
      nprow=2; npcol=3
! Set up a process grid of size nprow*npcol
      if (nprow*npcol.gt.nproc) then
         write(6,*) 'nproc = ',nproc,' less then nprow*npcol = '
      end if
      ctxt = ctxt_sys; call BLACS_GRIDINIT( ctxt, 'C', nprow, npcol)
! Processes not belonging to the grid jump to the end of program
      if (ctxt.lt.0) go to 1000
! Get the process coordinates in the grid
      call BLACS_GRIDINFO( ctxt, nprow, npcol, myrow, mycol )
! number of rows and columns of loacal parts al, bl for A, B
      m_al = NUMROC( n, nb, myrow, 0, nprow )
      n_al = NUMROC( n, nb, mycol, 0, npcol )
      m_bl = NUMROC( n, nb, myrow, 0, nprow )
      n_bl = NUMROC( 1, 1, mycol,0, npcol )
! Test for sufficient memory
      if (m_al*n_al.gt.max_al.or.
     :    m_bl*n_bl.gt.max_bl) then
       write(6,*)'not enough memory:  al ',m_al*n_al,max_al,
     :                               'bl', m_bl*n_bl,max_bl
       end if 
! set data for A and B; A(i,j) = 1./(mati+matj*(abs(i-j)), B(i,j) = bi*i+bj*j
      mati = 1.; matj = 5.; bi=1.; bj = 1.

! initializing descriptors for the distributed matrices A, B:
      llda = max(1,m_al); lldb = max(1,m_bl); lldc = max(1,m_cl)
      call DESCINIT( desc_A, n, n, nb, nb, 0, 0, ctxt, llda, info )
      call DESCINIT( desc_B, n, 1, nb, 1, 0, 0, ctxt, lldb, info )

! initialize in parallel the local parts of A in al and in al_s
      do jl = 1 , n_al
         j = INDXL2G(jl,nb,mycol,0,npcol)
         do il = 1 , m_al
            i = INDXL2G(il,nb,myrow,0,nprow)
            al(il+m_al*(jl-1))= 1.d0/(mati+matj*abs(i-j))
            al_s(il+m_al*(jl-1))= 1.d0/(mati+matj*abs(i-j))
         end do
      end do
! initialize in parallel the local parts of B in bl
      do il = 1 , m_bl
         i = INDXL2G(il,nb,myrow,0,nprow)
         bl(il)= bi*i + bj
      end do
!      call inspect(bl,'bl',desc_B) 

! solve the systems of equations
      call PDGESV( n, 1, al, 1, 1, desc_A, vl, bl, 1, 1, desc_B, info)
!      call inspect(bl,'x',desc_B)
! check the result
      ncheck = 10
         call BLACS_BARRIER(ctxt,'A')
         call sleep(1)
         if (myid.eq.0) write(6,*)'solution vector x'
         call BLACS_BARRIER(ctxt,'A')
         call sleep(1)
         jl = INDXG2L(1,1,mycol,0,npcol)
         ipc = INDXG2P(1,1,mycol,0,npcol)
         do i = 1 , min(ncheck,n)
            il = INDXG2L(i,nb,myrow,0,nprow)
            ipr = INDXG2P(i,nb,myrow,0,nprow)
            if (ipr.eq.myrow.and.ipc.eq.mycol) write(6,*)
     :             i,bl(il+m_bl*(jl-1))
         end do
         call BLACS_BARRIER(ctxt,'A')
         call sleep(1)
         if (myid.eq.0) write(6,*)'check of rhs vector'
         call BLACS_BARRIER(ctxt,'A')
         call sleep(1)
! matrix vector multiplication of coefficient matrix in al_s 
! times solutionvector in bl
! set up descriptor for resultvector R 
         call DESCINIT( desc_R, n, 1, n, 1,
     :               0, 0, ctxt, n, info )
         call PDGEMV( 'N',n,n,  1.d0,al_s,1,1,desc_A,
     :                               bl,1,1,desc_B,1,
     :                          0.d0,rl,1,1,desc_R,1, ) 
         err = 0.d0
         do i = 1 , n
            if (0.eq.myrow.and.0.eq.mycol.and.i.le.ncheck) write(6,*)
     :             i,rl(i), bi*i+bj
            err = err + abs(rl(i)- (bi*i+bj))
         end do
         call BLACS_BARRIER(ctxt,'A')
         call sleep(1)
         if (myid.eq.0) then
            write(6,*)'error in b-ax is',err/real(n)
         end if         

 1000  continue
      call BLACS_EXIT(0)

      stop
      end
