      program use_index_mapping
!     =========================
!  +-------------------------------------------------------------------+
!  |  Example program for initializing a block-cyclic distribted matrix|
!  |  as used in PBLAS and ScaLAPACK routines.                         |
!  |  In the exampel the block-cyclic distributed matrix needed        |
!  |  is generated directly by initializing the local arrays           |
!  !  in each process using the ScaLAPACK tool index mapping functions |       
!  |  INDXL2G, INDXG2L, INDXG2P                                        |
!  |                                                                   |
!  |  Paramters for the distributed matrix are                         |
!  |    dimension of matrix           m = 9,  n= 10                    |
!  |    blocksize for distribution    mb = 2, nb = 3                   |
!  |    grid dimensions               nprow = 3, npcol = 2             |
!  +-------------------------------------------------------------------+
      implicit none 
!  parameter constants
      integer       nmax
      parameter    (nmax = 1024)
!  local variables
      integer       ctxt, ctxt_0, ctxt_sys, ctxt_all,
     :              nproc, nprow, npcol, myid, myrow, mycol,
     :              desca(9), desca_0(9), NUMROC, ierr, info,
     :              m, n, mb, nb, ml, nl, lld, ipr, ipc, i, j, il, jl,
     :              INDXL2G, INDXG2L, INDXG2P
      real*8        matval, bl(nmax*nmax), cl(nmax*nmax)

!  calling BLACS_PINFO(myid,nproc) initializes MPI and returns size and rank
      call BLACS_PINFO(myid,nproc) 

!  set parameters for the distribtued matrix
      m = 9;  n= 10            
      mb = 2; nb = 3     
      nprow = 3; npcol = 2 

! get the system default context
      CALL BLACS_GET( 0, 0, ctxt_sys )

! Set up a process grid ctxt_all of size nproc 
      ctxt_all = ctxt_sys
      CALL BLACS_GRIDINIT( ctxt_all, 'C', nproc, 1)

! Set up a process grid ctxt of size nprow*npcol
      ctxt = ctxt_sys
      CALL BLACS_GRIDINIT( ctxt, 'C', nprow, npcol)

! All processes not belonging to ctxt jump to the end of the program
      if (ctxt.lt.0) go to 1000

! Get the process coordinates in the grid with contet ctxt
      CALL BLACS_GRIDINFO( ctxt, nprow, npcol, myrow, mycol )

! Initializing the descriptor for the distributed mxn matrix a:
      ml = NUMROC( m, mb, myrow, 0, nprow )
      nl = NUMROC( n, nb, mycol, 0, npcol )
      lld =  max(1,ml)
      call DESCINIT( desca, m, n, mb, nb, 0, 0, ctxt, lld, info )
      if (info.ne.0) write(6,*)'desca :',myid,info

! copy the values of the global matrix
! to the local parts bl of the distributed matrix
      do j = 1 , n
         jl = INDXG2L(j,nb,0,0,npcol)
         ipc = INDXG2P(j,nb,0,0,npcol)
         do i = 1 , m
            il = INDXG2L(i,mb,0,0,nprow)
            ipr = INDXG2P(i,mb,0,0,nprow)
            if (ipr.eq.myrow.and.ipc.eq.mycol)
     :             bl(il+ml*(jl-1))= matval(i,j)
         end do
      end do

! initialize in parallel the local parts cl of the distributed matrix
      ml = NUMROC( m, mb, myrow, 0, nprow )
      nl = NUMROC( n, nb, mycol, 0, npcol )
      do jl = 1 , nl
         j = INDXL2G(jl,nb,mycol,0,npcol)
         do il = 1 , ml
            i = INDXL2G(il,mb,myrow,0,nprow)
            cl(il+ml*(jl-1))= matval(i,j)
         end do
      end do

! Inspect the local elements of the distributed matrix on each process
      call BLACS_BARRIER(ctxt,'A')
      call sleep(1)
      if (m.le.10.and.n.le.10) then
      do ipc = 0 , npcol-1
         do ipr = 0, nprow-1
            if (ipr.eq.myrow.and.ipc.eq.mycol.and.n.le.10) then
                write(6,232) ipr, ipc
                ml = NUMROC( m, mb, myrow, 0, nprow )
                nl = NUMROC( n, nb, mycol, 0, npcol )
                do i = 1 , ml
                   write(6,233)(bl(i+ml*(j-1)),j = 1,nl)
                   write(6,233)(cl(i+ml*(j-1)),j = 1,nl)
                end do
 232   format('local arrays bl, cl on process (',i2,','i2,')')
 233   format(10f8.0)           
            end if
            call BLACS_BARRIER(ctxt,'A')
            call sleep(1)
         end do
      end do
      end if
      call BLACS_BARRIER(ctxt,'A')
 1000  continue
      call BLACS_EXIT(0)

      stop
      end

      real*8 function matval(i,j)
      implicit none
      integer i,j
      matval = 100*i + j
      return 
      end
