      program piprec_mpi_omp
      implicit none
      include 'mpif.h'
      include 'omp_lib.h'

      integer  ierr, np, me, tid, comm, tp
      double precision farmer, pia, t, r, piexact
      integer  idle, n, ndom, ipfa
      piexact = 3.14159265358979323846264338327950288419d0 
      idle = 1


!  initialize MPI, get nproc (# of procs) and myid (local proc. id)
      comm = MPI_COMM_WORLD
      call MPI_INIT_THREAD(MPI_THREAD_MULTIPLE,tp, ierr )
      call MPI_COMM_SIZE( comm, np, ierr )
      call MPI_COMM_RANK( comm, me, ierr )
      ipfa = 0
      if (me.eq.ipfa) then
         call MPI_QUERY_THREAD(tp,ierr)
         write(6,*)'thread support = ', tp
!  input of strips and number of intervals 
         n = 1000000000
         ndom = 20
         write(6,*)'number of strips ?'
         read(5,*) n
         write(6,*)'number of intervals ?'
         read(5,*) ndom
         write(6,1001) n, ndom
 1001 format(//'calculation of pi using numerical integration'
     :         'Integration von SQRT(1-x**2)'
     :        /'number of strips       : ',i12,
     :        /'number of subintervals:  : ',i8)

      end if

      call MPI_BCAST(n,1,MPI_INTEGER,ipfa,comm,ierr)
      call MPI_BCAST(ndom,1,MPI_INTEGER,ipfa,comm,ierr)

! ---------------------------  worker  -----------------------     
      if (me.ne.ipfa) then
         call MPI_BARRIER(comm,ierr)
         call worker(ipfa,n,ndom)
      end if

! ---------------------------  farmer  -----------------------     
      if (me.eq.ipfa) then
         call OMP_SET_NUM_THREADS(2)
!$OMP PARALLEL DEFAULT(SHARED) PRIVATE(tid)
         tid = OMP_GET_THREAD_NUM()
! ---------------------------  farmer-worker  -----------------------     
         if (tid.eq.1) then
            call worker(ipfa,n,ndom)
         end if     
! ---------------------------  farmer-farmer  -----------------------     
         if (tid.eq.0) then
         call MPI_BARRIER(comm,ierr)
            t = MPI_Wtime()
            pia = farmer(idle,ndom)
            t = MPI_Wtime() - t
         end if
!$OMP END PARALLEL
         r = dble(n)/t*1.e-6
         write(6,1111)piexact, pia, abs(pia-piexact)
 1111 format(/'pi exact          = ',f25.18,
     :       /'pi estimated      = ',f25.18,
     :       //'error            = ',e20.3)
         print  '(a,e11.4,a,f8.2,a)'
     :          ,'time : ',t,' s , rate :',r,' [Mstrips/s]'
      end if
      call MPI_FINALIZE( ierr )
      stop
      end

